@@ -3,7 +3,7 @@
  *
  * minimodem - software audio Bell-type or RTTY FSK modem
  *
- * Copyright (C) 2011-2014 Kamal Mostafa <kamal@whence.com>
+ * Copyright (C) 2011-2016 Kamal Mostafa <kamal@whence.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <assert.h>
 #include <signal.h>
 #include <sys/time.h>
+#include <sys/select.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -119,12 +120,12 @@ static void fsk_transmit_stdin(
 	int bfsk_msb_first,
 	unsigned int bfsk_do_tx_sync_bytes,
 	unsigned int bfsk_sync_byte,
-	databits_encoder encode
+	databits_encoder encode,
+	int txcarrier
 	)
 {
     size_t sample_rate = simpleaudio_get_rate(sa_out);
-    size_t bit_nsamples = sample_rate / data_rate + 0.5;
-    int c;
+    size_t bit_nsamples = sample_rate / data_rate + 0.5f;
 
     tx_sa_out = sa_out;
     tx_bfsk_mark_f = bfsk_mark_f;
@@ -133,7 +134,7 @@ static void fsk_transmit_stdin(
     // one-shot
     struct itimerval itv = {
 	{0, 0},						// it_interval
-	{0, 1000000/(float)(data_rate+data_rate*0.03) }	// it_value
+	{0, 1000000/(float)(data_rate+data_rate*0.03f)}	// it_value
     };
 
     struct itimerval itv_zero = {
@@ -141,44 +142,85 @@ static void fsk_transmit_stdin(
 	{0, 0}						// it_value
     };
 
-    if ( tx_interactive )
+    int block_input = tx_interactive && !txcarrier;
+    if ( block_input )
 	signal(SIGALRM, tx_stop_transmit_sighandler);
 
+    // Set up for select() should we need it
+    int fd = fileno(stdin);
+    fd_set fdset;
+
     tx_transmitting = 0;
-    while ( (c = getchar()) != EOF )
+    int end_of_file = 0;
+    unsigned char buf;
+    int n_read = 0;
+    int idle = 0;
+    while ( !end_of_file )
     {
-	if ( tx_interactive )
+        FD_ZERO(&fdset);
+        FD_SET(fd, &fdset);
+        struct timeval tv_zero = { 0, 0 };
+        if( block_input || select(fd+1, &fdset, NULL, NULL, &tv_zero) )
+        {
+	    n_read = read(fd, &buf, sizeof(buf));
+	    if( n_read <= 0 ) //Includes EOF (0) and errors (-1)
+	    {
+		end_of_file = 1;
+		continue;     //Do nothing else
+	    }
+            idle = 0;
+        }
+	else
+	    idle = 1;
+
+	// Cause any running timer to immediately trigger
+	if ( block_input )
 	    setitimer(ITIMER_REAL, &itv_zero, NULL);
 
-	// fprintf(stderr, "<c=%d>", c);
-	unsigned int nwords;
-	unsigned int bits[2];
-	unsigned int j;
-	nwords = encode(bits, c);
-
-	if ( !tx_transmitting )
+	if( !idle )
 	{
-	    tx_transmitting = 1;
-	    /* emit leader tone (mark) */
-	    for ( j=0; j<tx_leader_bits_len; j++ )
-		simpleaudio_tone(sa_out, invert_start_stop ? bfsk_space_f : bfsk_mark_f, bit_nsamples);
-	    /* emit "preamble" of sync bytes */
-	    for ( j=0; j<bfsk_do_tx_sync_bytes; j++ )
-		fsk_transmit_frame(sa_out, bfsk_sync_byte, n_data_bits,
+	    // fprintf(stderr, "<c=%d>", c);
+	    unsigned int nwords;
+	    unsigned int bits[2];
+	    unsigned int j;
+	    nwords = encode(bits, buf);
+
+	    if ( !tx_transmitting )
+	    {
+	        tx_transmitting = 1;
+                /* emit leader tone (mark) */
+                for ( j=0; j<tx_leader_bits_len; j++ )
+                    simpleaudio_tone(sa_out, invert_start_stop ? bfsk_space_f : bfsk_mark_f, bit_nsamples);
+	    }
+	    if ( tx_transmitting < 2)
+	    {
+		tx_transmitting = 2;
+		/* emit "preamble" of sync bytes */
+		for ( j=0; j<bfsk_do_tx_sync_bytes; j++ )
+		    fsk_transmit_frame(sa_out, bfsk_sync_byte, n_data_bits,
 			    bit_nsamples, bfsk_mark_f, bfsk_space_f,
 			    bfsk_nstartbits, bfsk_nstopbits, invert_start_stop, 0);
+	    }
+
+	    /* emit data bits */
+	    for ( j=0; j<nwords; j++ )
+		fsk_transmit_frame(sa_out, bits[j], n_data_bits,
+			    bit_nsamples, bfsk_mark_f, bfsk_space_f,
+			    bfsk_nstartbits, bfsk_nstopbits, invert_start_stop, bfsk_msb_first);
+        }
+        else
+        {
+	    tx_transmitting = 1;
+	    unsigned int j;
+            /* emit idle tone (mark) */
+            for ( j=0; j<tx_leader_bits_len; j++ )
+                simpleaudio_tone(sa_out, invert_start_stop ? bfsk_space_f : bfsk_mark_f, sample_rate/50);
 	}
 
-	/* emit data bits */
-	for ( j=0; j<nwords; j++ )
-	    fsk_transmit_frame(sa_out, bits[j], n_data_bits,
-			bit_nsamples, bfsk_mark_f, bfsk_space_f,
-			bfsk_nstartbits, bfsk_nstopbits, invert_start_stop, bfsk_msb_first);
-
-	if ( tx_interactive )
+	if ( block_input )
 	    setitimer(ITIMER_REAL, &itv, NULL);
     }
-    if ( tx_interactive ) {
+    if ( block_input ) {
 	setitimer(ITIMER_REAL, &itv_zero, NULL);
 	signal(SIGALRM, SIG_DFL);
     }
@@ -209,21 +251,21 @@ report_no_carrier( fsk_plan *fskp,
 		nbits_decoded * sample_rate / (float)carrier_nsamples;
     fprintf(stderr, "\n### NOCARRIER ndata=%u confidence=%.3f ampl=%.3f bps=%.2f",
 	    nframes_decoded,
-	    confidence_total / nframes_decoded,
-	    amplitude_total / nframes_decoded,
-	    throughput_rate);
+	    (double)(confidence_total / nframes_decoded),
+	    (double)(amplitude_total / nframes_decoded),
+	    (double)(throughput_rate));
 #if 0
     fprintf(stderr, " bits*sr=%llu rate*nsamp=%llu",
 	    (unsigned long long)(nbits_decoded * sample_rate + 0.5),
 	    (unsigned long long)(bfsk_data_rate * carrier_nsamples) );
 #endif
-    if ( (unsigned long long)(nbits_decoded * sample_rate + 0.5) == (unsigned long long)(bfsk_data_rate * carrier_nsamples) ) {
+    if ( (unsigned long long)(nbits_decoded * sample_rate + 0.5f) == (unsigned long long)(bfsk_data_rate * carrier_nsamples) ) {
 	fprintf(stderr, " (rate perfect) ###\n");
     } else {
 	float throughput_skew = (throughput_rate - bfsk_data_rate)
 			    / bfsk_data_rate;
 	fprintf(stderr, " (%.1f%% %s) ###\n",
-		fabs(throughput_skew) * 100.0,
+		(double)(fabsf(throughput_skew) * 100.0f),
 		signbit(throughput_skew) ? "slow" : "fast"
 		);
     }
@@ -248,7 +290,8 @@ benchmarks()
 
     int ret;
     ret = system("sed -n -e '/^model name/{p;q}' -e '/^cpu model/{p;q}' /proc/cpuinfo");
-    ret = ret;	// don't care, hush compiler.
+    if ( ret )
+	;	// don't care, hush compiler.
 
     fflush(stdout);
 
@@ -317,7 +360,7 @@ version()
 {
     printf(
     "minimodem %s\n"
-    "Copyright (C) 2011-2014 Kamal Mostafa <kamal@whence.com>\n"
+    "Copyright (C) 2011-2016 Kamal Mostafa <kamal@whence.com>\n"
     "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n"
     "This is free software: you are free to change and redistribute it.\n"
     "There is NO WARRANTY, to the extent permitted by law.\n\n"
@@ -358,12 +401,15 @@ usage()
     "		    --rx-one\n"
     "		    --benchmarks\n"
     "		    --binary-output\n"
+    "		    --binary-raw {nbits}\n"
     "		    --print-filter\n"
+    "		    --tx-carrier\n"
     "		{baudmode}\n"
     "	    any_number_N       Bell-like      N bps --ascii\n"
     "		    1200       Bell202     1200 bps --ascii\n"
     "		     300       Bell103      300 bps --ascii\n"
     "		    rtty       RTTY       45.45 bps --baudot --stopbits=1.5\n"
+    "		     tdd       TTY/TDD    45.45 bps --baudot --stopbits=2.0\n"
     "		    same       NOAA SAME 520.83 bps --sync-byte=0xAB ...\n"
     "		callerid       Bell202 CID 1200 bps\n"
     "     uic{-train,-ground}       UIC-751-3 Train/Ground 600 bps\n"
@@ -399,7 +445,7 @@ build_expect_bits_string( char *expect_bits_string,
 	char start_bit_value = invert_start_stop ? '1' : '0';
 	char stop_bit_value = invert_start_stop ? '0' : '1';
 	int j = 0;
-	if ( bfsk_nstopbits != 0.0 )
+	if ( bfsk_nstopbits != 0.0f )
 	    expect_bits_string[j++] = stop_bit_value;
 	int i;
 	// Nb. only integer number of start bits works (for rx)
@@ -411,7 +457,7 @@ build_expect_bits_string( char *expect_bits_string,
 	    else
 		expect_bits_string[j] = 'd';
 	}
-	if ( bfsk_nstopbits != 0.0 )
+	if ( bfsk_nstopbits != 0.0f )
 	    expect_bits_string[j++] = stop_bit_value;
 	expect_bits_string[j] = 0;
 
@@ -426,8 +472,8 @@ main( int argc, char*argv[] )
     int quiet_mode = 0;
     int output_print_filter = 0;
     float band_width = 0;
-    unsigned int bfsk_mark_f = 0;
-    unsigned int bfsk_space_f = 0;
+    float bfsk_mark_f = 0;
+    float bfsk_space_f = 0;
     unsigned int bfsk_inverted_freqs = 0;
     int bfsk_nstartbits = -1;
     float bfsk_nstopbits = -1;
@@ -472,7 +518,10 @@ main( int argc, char*argv[] )
     unsigned int rx_one = 0;
     float rxnoise_factor = 0.0;
 
+    int txcarrier = 0;
+
     int output_mode_binary = 0;
+    int output_mode_raw_nbits = 0;
 
     float	bfsk_data_rate = 0.0;
     databits_encoder	*bfsk_databits_encode;
@@ -510,8 +559,10 @@ main( int argc, char*argv[] )
 	MINIMODEM_OPT_RX_ONE,
 	MINIMODEM_OPT_BENCHMARKS,
 	MINIMODEM_OPT_BINARY_OUTPUT,
+	MINIMODEM_OPT_BINARY_RAW,
 	MINIMODEM_OPT_PRINT_FILTER,
 	MINIMODEM_OPT_XRXNOISE,
+	MINIMODEM_OPT_TXCARRIER
     };
 
     while ( 1 ) {
@@ -548,8 +599,10 @@ main( int argc, char*argv[] )
 	    { "rx-one",		0, 0, MINIMODEM_OPT_RX_ONE },
 	    { "benchmarks",	0, 0, MINIMODEM_OPT_BENCHMARKS },
 	    { "binary-output",	0, 0, MINIMODEM_OPT_BINARY_OUTPUT },
+	    { "binary-raw",	1, 0, MINIMODEM_OPT_BINARY_RAW },
 	    { "print-filter",	0, 0, MINIMODEM_OPT_PRINT_FILTER },
 	    { "Xrxnoise",	1, 0, MINIMODEM_OPT_XRXNOISE },
+	    { "tx-carrier",      0, 0, MINIMODEM_OPT_TXCARRIER },
 	    { 0 }
 	};
 	c = getopt_long(argc, argv, "Vtrc:l:ai875f:b:v:M:S:T:qA::R:",
@@ -608,14 +661,14 @@ main( int argc, char*argv[] )
 			    tx_amplitude = FLT_EPSILON;
 			else
 			    tx_amplitude = atof(optarg);
-			assert( tx_amplitude > 0.0 );
+			assert( tx_amplitude > 0.0f );
 			break;
 	    case 'M':
-			bfsk_mark_f = atoi(optarg);
+			bfsk_mark_f = atof(optarg);
 			assert( bfsk_mark_f > 0 );
 			break;
 	    case 'S':
-			bfsk_space_f = atoi(optarg);
+			bfsk_space_f = atof(optarg);
 			assert( bfsk_space_f > 0 );
 			break;
 	    case MINIMODEM_OPT_STARTBITS:
@@ -669,12 +722,18 @@ main( int argc, char*argv[] )
 	    case MINIMODEM_OPT_BINARY_OUTPUT:
 			output_mode_binary = 1;
 			break;
+	    case MINIMODEM_OPT_BINARY_RAW:
+			output_mode_raw_nbits = atoi(optarg);
+			break;
 	    case MINIMODEM_OPT_PRINT_FILTER:
 			output_print_filter = 1;
 			break;
 	    case MINIMODEM_OPT_XRXNOISE:
 			rxnoise_factor = atof(optarg);
 			break;
+	    case MINIMODEM_OPT_TXCARRIER:
+			txcarrier = 1;
+			break;
 	    default:
 			usage();
 	}
@@ -723,6 +782,16 @@ main( int argc, char*argv[] )
 	    bfsk_n_data_bits = 5;
 	if ( bfsk_nstopbits < 0 )
 	    bfsk_nstopbits = 1.5;
+    } else if ( strncasecmp(modem_mode, "tdd",3)==0 ) {
+	bfsk_databits_decode = databits_decode_baudot;
+	bfsk_databits_encode = databits_encode_baudot;
+	bfsk_data_rate = 45.45;
+	if ( bfsk_n_data_bits == 0 )
+	    bfsk_n_data_bits = 5;
+	if ( bfsk_nstopbits < 0 )
+	    bfsk_nstopbits = 2.0;
+	bfsk_mark_f = 1400;
+	bfsk_space_f = 1800;
     } else if ( strncasecmp(modem_mode, "same",5)==0 ) {
 	// http://www.nws.noaa.gov/nwr/nwrsame.htm
 	bfsk_data_rate = 520.0 + 5/6.0;
@@ -740,7 +809,7 @@ main( int argc, char*argv[] )
 	    fprintf(stderr, "E: callerid --tx mode is not supported.\n");
 	    return 1;
 	}
-	if ( carrier_autodetect_threshold > 0.0 )
+	if ( carrier_autodetect_threshold > 0.0f )
 	    fprintf(stderr, "W: callerid with --auto-carrier is not recommended.\n");
 	bfsk_databits_decode = databits_decode_callerid;
 	bfsk_data_rate = 1200;
@@ -768,13 +837,19 @@ main( int argc, char*argv[] )
 	if ( bfsk_n_data_bits == 0 )
 	    bfsk_n_data_bits = 8;
     }
-    if ( bfsk_data_rate == 0.0 )
+    if ( bfsk_data_rate == 0.0f )
 	usage();
 
 
-    if ( output_mode_binary )
+    if ( output_mode_binary || output_mode_raw_nbits )
 	bfsk_databits_decode = databits_decode_binary;
 
+    if ( output_mode_raw_nbits ) {
+	bfsk_nstartbits = 0;
+	bfsk_nstopbits = 0;
+	bfsk_n_data_bits = output_mode_raw_nbits;
+    }
+
     if ( bfsk_data_rate >= 400 ) {
 	/*
 	 * Bell 202:     baud=1200 mark=1200 space=2200
@@ -874,7 +949,8 @@ main( int argc, char*argv[] )
 				bfsk_msb_first,
 				bfsk_do_tx_sync_bytes,
 				bfsk_sync_byte,
-				bfsk_databits_encode
+				bfsk_databits_encode,
+				txcarrier
 				);
 
 	simpleaudio_close(sa_out);
@@ -899,7 +975,7 @@ main( int argc, char*argv[] )
 
     sample_rate = simpleaudio_get_rate(sa);
 
-    if ( rxnoise_factor != 0.0 )
+    if ( rxnoise_factor != 0.0f )
 	simpleaudio_set_rxnoise(sa, rxnoise_factor);
 
     /*
@@ -970,19 +1046,19 @@ main( int argc, char*argv[] )
     // for encodings without start/stop bits:
     //     MUST be <= 0.5 or we may accidentally skip a bit
     //
-    assert( fsk_frame_overscan >= 0.0 && fsk_frame_overscan < 1.0 );
+    assert( fsk_frame_overscan >= 0.0f && fsk_frame_overscan < 1.0f );
 
     // ensure that we overscan at least a single sample
     unsigned int nsamples_overscan
-			= nsamples_per_bit * fsk_frame_overscan + 0.5;
-    if ( fsk_frame_overscan > 0.0 && nsamples_overscan == 0 )
+			= nsamples_per_bit * fsk_frame_overscan + 0.5f;
+    if ( fsk_frame_overscan > 0.0f && nsamples_overscan == 0 )
 	nsamples_overscan = 1;
     debug_log("fsk_frame_overscan=%f nsamples_overscan=%u\n",
 	    fsk_frame_overscan, nsamples_overscan);
 
     // n databits plus bfsk_startbit start bits plus bfsk_nstopbit stop bits:
     float frame_n_bits = bfsk_n_data_bits + bfsk_nstartbits + bfsk_nstopbits;
-    unsigned int frame_nsamples = nsamples_per_bit * frame_n_bits + 0.5;
+    unsigned int frame_nsamples = nsamples_per_bit * frame_n_bits + 0.5f;
 
     char expect_data_string_buffer[64];
     if (expect_data_string == NULL) {
@@ -1050,7 +1126,7 @@ main( int argc, char*argv[] )
 
 	/* Auto-detect carrier frequency */
 	static int carrier_band = -1;
-	if ( carrier_autodetect_threshold > 0.0 && carrier_band < 0 ) {
+	if ( carrier_autodetect_threshold > 0.0f && carrier_band < 0 ) {
 	    unsigned int i;
 	    float nsamples_per_scan = nsamples_per_bit;
 	    if ( nsamples_per_scan > fskp->fftsize )
@@ -1072,7 +1148,7 @@ main( int argc, char*argv[] )
 	    }
 
 	    // default negative shift -- reasonable?
-	    int b_shift = - (float)(autodetect_shift + fskp->band_width/2.0)
+	    int b_shift = - (float)(autodetect_shift + fskp->band_width/2.0f)
 						/ fskp->band_width;
 	    if ( bfsk_inverted_freqs )
 		b_shift *= -1;
@@ -1107,7 +1183,7 @@ main( int argc, char*argv[] )
 	// 2. allows us to track slightly slow signals
 	unsigned int try_max_nsamples;
 	if ( carrier )
-	    try_max_nsamples = nsamples_per_bit * 0.75 + 0.5;
+	    try_max_nsamples = nsamples_per_bit * 0.75f + 0.5f;
 	else
 	    try_max_nsamples = nsamples_per_bit;
 	try_max_nsamples += nsamples_overscan;
@@ -1147,7 +1223,7 @@ main( int argc, char*argv[] )
 
 	int do_refine_frame = 0;
 
-	if ( confidence < peak_confidence * 0.75 ) {
+	if ( confidence < peak_confidence * 0.75f ) {
 	    do_refine_frame = 1;
 	    debug_log(" ... do_refine_frame rescan (confidence %.3f << %.3f peak)\n", confidence, peak_confidence);
 	    peak_confidence = 0;
@@ -1155,7 +1231,7 @@ main( int argc, char*argv[] )
 
 	// no-confidence if amplitude drops abruptly to < 25% of the
 	// track_amplitude, which follows amplitude with hysteresis
-	if ( amplitude < track_amplitude * 0.25 ) {
+	if ( amplitude < track_amplitude * 0.25f ) {
 	    confidence = 0;
 	}
 
@@ -1208,12 +1284,12 @@ main( int argc, char*argv[] )
 	    if ( !quiet_mode ) {
 		if ( bfsk_data_rate >= 100 )
 		    fprintf(stderr, "### CARRIER %u @ %.1f Hz ",
-			    (unsigned int)(bfsk_data_rate + 0.5),
-			    fskp->b_mark * fskp->band_width);
+			    (unsigned int)(bfsk_data_rate + 0.5f),
+			    (double)(fskp->b_mark * fskp->band_width));
 		else
 		    fprintf(stderr, "### CARRIER %.2f @ %.1f Hz ",
-			    bfsk_data_rate,
-			    fskp->b_mark * fskp->band_width);
+			    (double)(bfsk_data_rate),
+			    (double)(fskp->b_mark * fskp->band_width));
 	    }
 
 	    if ( !quiet_mode )
@@ -1284,7 +1360,7 @@ main( int argc, char*argv[] )
 		    frame_start_sample, advance);
 
 	// chop off the prev_stop bit
-	if ( bfsk_nstopbits != 0.0 )
+	if ( bfsk_nstopbits != 0.0f )
 	    bits = bits >> 1;
 
 
